#############################################################################
# Copyright (c) 2021, nb_cron Contributors                                  #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################
# flake8: noqa
from .handlers import load_jupyter_server_extension
from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

def _jupyter_nbextension_paths():
    return [dict(section="notebook",
                 src="static",
                 dest="nb_cron",
                 require="nb_cron/notebook"),
            dict(section="tree",
                 src="static",
                 dest="nb_cron",
                 require="nb_cron/tree")]


def _jupyter_server_extension_paths():
    return [dict(module="nb_cron")]




#############################################################################
# Copyright (c) 2021, nb_cron Contributors                                  #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################
import sys

import requests
from notebook.nbextensions import enable_nbextension_python, install_nbextension_python
from notebook.serverextensions import toggle_serverextension_python
from notebook.tests.launchnotebook import NotebookTestBase
from notebook.utils import url_path_join
from traitlets.config.loader import Config

import nb_cron


class NbCronAPI(object):
    """Wrapper for nbconvert API calls."""

    def __init__(self, base_url, token):
        self.base_url = str(base_url)
        self.token = str(token)

    def _req(self, verb, path, body=None, params=None):
        if body is None:
            body = {}

        session = requests.session()
        resp = session.get(self.base_url + '?token=' + self.token, allow_redirects=True)
        xsrf_token = None
        if '_xsrf' in session.cookies:
            xsrf_token = session.cookies['_xsrf']
        body.update({'_xsrf': xsrf_token})
        response = session.request(
            verb,
            url_path_join(self.base_url, 'cron', *path),
            data=body, params=params,
        )
        return response

    def get(self, path, body=None, params=None):
        return self._req('GET', path, body, params)

    def post(self, path, body=None, params=None):
        return self._req('POST', path, body, params)

    def jobs(self):
        res = self.get(["jobs"])
        if res is not None:
            return res.json()
        else:
            return {}


class NbCronAPITest(NotebookTestBase):
    def setUp(self):
        if 'nb_cron' not in sys.modules:
            sys.modules['nb_cron'] = nb_cron
        c = Config()
        c.NotebookApp.nbserver_extensions = {}
        c.NotebookApp.nbserver_extensions.update({'nb_cron': True})
        c.NotebookApp.allow_origin = '*'
        c.NotebookApp.allow_credentials = True
        c.NotebookApp.disable_check_xsrf = True
        self.config = c
        install_nbextension_python("nb_cron", user=True)
        enable_nbextension_python("nb_cron")
        toggle_serverextension_python("nb_cron", True)
        super(NbCronAPITest, self).setUp()
        self.__class__.notebook.init_server_extension_config()
        self.__class__.notebook.init_server_extensions()

        # chrome_options = Options()
        # # chrome_options.add_argument("--headless")
        # self.driver = webdriver.Chrome(options=chrome_options)
        # self.driver.get(self.base_url() + '?token=' + self.token)
        # self.driver.implicitly_wait(120)  # seconds
        # import time
        # time.sleep(60)

        self.cron_api = NbCronAPI(self.base_url(), self.token)
        self.job_schedule = "* * * * *"
        self.job_command = "echo"
        self.job_comment = "comment"
        self.notebook_path = "tests/python parameter test.ipynb"
        self.create_job()
        self.job_id = len(self.cron_api.jobs()) - 1

    def tearDown(self):
        # self.driver.quit()
        self.remove_job(self.job_id)
        super(NbCronAPITest, self).tearDown()

    def create_job(self, schedule=None, command=None, comment=None):
        return self.cron_api.post(["jobs", str(-1),
                                   "create"],
                                  params={"schedule": schedule or self.job_schedule,
                                          "command": command or self.job_command,
                                          "comment": comment or self.job_comment})

    def remove_job(self, jid=None):
        return self.cron_api.post(["jobs", str(jid or self.job_id),
                                   "remove"])

    def edit_job(self, jid=None, schedule=None, command=None, comment=None):
        return self.cron_api.post(["jobs", str(jid or self.job_id),
                                   "edit"],
                                  params={"schedule": schedule or self.job_schedule,
                                          "command": command or self.job_command,
                                          "comment": comment or self.job_comment})

    def check_schedule(self, schedule=None):
        return self.cron_api.post(["schedule", "check"],
                                  params={"schedule": schedule or self.job_schedule})

    def extract_papermill_parameters(self, notebook_path=None):
        return self.cron_api.post(["notebook", "papermill"],
                                  params={"path": notebook_path or self.notebook_path})

    def test_01_job_list(self):
        jobs = self.cron_api.jobs()
        root = filter(lambda job: job["schedule"] == "* * * * *",
                      jobs["jobs"])
        self.assertGreaterEqual(len(list(root)), 1)

    def test_02_job_create_and_remove(self):
        self.assertEqual(self.create_job().status_code, 201)
        jid = len(self.cron_api.jobs()) - 1
        self.assertEqual(self.remove_job(jid).status_code, 200)

    def test_03_job_create_fail(self):
        self.assertEqual(self.create_job(schedule=" ").status_code, 422)
        self.assertEqual(self.create_job(schedule="* * * * * *").status_code, 422)
    
    def test_04_job_remove_fail(self):
        self.assertEqual(self.remove_job(' ').status_code, 404)
        self.assertEqual(self.remove_job(-2).status_code, 404)
        self.assertEqual(self.remove_job(999999).status_code, 422)

    def test_05_job_create_edit_remove(self):
        self.assertEqual(self.create_job().status_code, 201)
        jid = len(self.cron_api.jobs()) - 1
        self.assertEqual(self.edit_job(jid, command='echo edit test').status_code, 200)
        self.assertEqual(self.remove_job(jid).status_code, 200)

    def test_06_job_edit_fail(self):
        self.assertEqual(self.edit_job(jid=" ").status_code, 404)
        self.assertEqual(self.edit_job(jid=8888).status_code, 422)
        self.assertEqual(self.edit_job(command=" ").status_code, 422)
        self.assertEqual(self.edit_job(schedule=" ").status_code, 422)
        self.assertEqual(self.edit_job(schedule="* * * * * *").status_code, 422)

    def test_07_job_nonsense(self):
        r = self.cron_api.post(["jobs", str(self.job_id), "nonsense"])
        self.assertEqual(r.status_code, 404)

    def test_08_schedule_check(self):
        self.assertEqual(self.check_schedule().status_code, 200)

    def test_09_schedule_check_fail(self):
        self.assertEqual(self.check_schedule(schedule=' ').status_code, 422)
        self.assertEqual(self.check_schedule(schedule='* * * * * *').status_code, 422)

    def test_10_extract_papermill_parameters(self):
        self.assertEqual(self.extract_papermill_parameters(notebook_path='tests/python parameter test.ipynb').status_code, 200)
        self.assertEqual(self.extract_papermill_parameters(notebook_path='tests/spark parameter test.ipynb').status_code, 200)
        self.assertEqual(self.extract_papermill_parameters(notebook_path='tests/pyspark parameter test.ipynb').status_code, 200)

    def test_11_extract_papermill_parameters_fail(self):
        self.assertEqual(self.extract_papermill_parameters(notebook_path=' ').status_code, 422)
        self.assertEqual(self.extract_papermill_parameters(notebook_path='test.ipynb').status_code, 422)



#############################################################################
# Copyright (c) 2021, nb_cron Contributors                                  #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################
import datetime
import json

from crontab import CronTab
from traitlets import Dict, Unicode
from traitlets.config.configurable import LoggingConfigurable


class JobManager(LoggingConfigurable):
    conda_prefix_local = Unicode(
        help="prefix to detect conda in local environment", allow_none=True, default_value='conda-env-.conda-'
    ).tag(config=True, env="NBCRON_CONDA_PREFIX_LOCAL")
    conda_prefix_global = Unicode(
        help="prefix to detect conda in global environment", allow_none=True, default_value='conda-env-'
    ).tag(config=True, env="NBCRON_CONDA_PREFIX_GLOBAL")
    conda_separator = Unicode(
        help="separator between env and kernel name", allow_none=True, default_value='-'
    ).tag(config=True, env="NBCRON_CONDA_SEPARATOR")

    jobs = Dict()

    def list_jobs(self):
        """List all cron jobs"""
        cron = CronTab(user=True)
        jobs = []
        for i in range(len(cron)):
            jobs.append({
                'id': i,
                'schedule': str(cron[i].slices),
                'command': str(cron[i].command),
                'comment': str(cron[i].comment)
            })

        self.log.debug("jobs: %s" % str(jobs))

        return {
            "jobs": jobs,
            "status_code": 200
        }

    def remove_job(self, job):
        cron = CronTab(user=True)
        try:
            self.log.debug('deleting cron job id %s', job)
            cron.remove(cron[job])
            cron.write()
        except Exception as err:
            self.log.error('[nb_cron] Job delete fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }

        return {'status_code': 200}

    def create_job(self, schedule, command, comment):
        cron = CronTab(user=True)
        try:
            self.log.debug('creating cron job schedule:%s command:%s comment:%s',
                           schedule, command, comment)
            job = cron.new(command=command, comment=comment, pre_comment=True)
            job.setall(schedule)
            if not job.is_valid():
                return {
                    "error": True,
                    "message": u"Job is invalid.",
                    "status_code": 422
                }
            cron.write()
        except KeyError as err:
            self.log.error('[nb_cron] Job create fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }

        return {
            'id': len(cron) - 1,
            'status_code': 200
        }

    def edit_job(self, job, schedule, command, comment):
        cron = CronTab(user=True)

        # check if job id is within range
        if job < 0 or job > len(cron):
            return {
                "error": True,
                "message": u"Job id not found.",
                "status_code": 422
            }

        job = cron[job]
        try:
            self.log.debug('editing cron job id:%s schedule:%s command:%s comment:%s',
                           str(job), schedule, command, comment)
            job.set_command(command)
            job.set_comment(comment, pre_comment=True)
            job.setall(schedule)
            if not job.is_valid():
                return {
                    "error": True,
                    "message": u"Job is invalid.",
                    "status_code": 422
                }
            cron.write()
        except KeyError as err:
            self.log.error('[nb_cron] Job edit fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }

        return {'status_code': 200}

    def check_schedule(self, schedule):
        """List next 5 schedule"""
        cron = CronTab(user=True)
        job = cron.new(command='')
        try:
            job.setall(schedule)
        except KeyError as err:
            self.log.debug('[nb_cron] Schedule check fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }

        sch = job.schedule(date_from=datetime.datetime.now())
        schedules = []
        for i in range(5):
            schedules.append(str(sch.get_next()))

        return {
            "schedules": schedules,
            "status_code": 200
        }

    def extract_variables(self, _nb_cron_code, _nb_cron_kernel='python'):
        """Extract declared variables from cell

        Args:
            _nb_cron_code: cell code
            _nb_cron_kernel: kernel of code for preprocessing

        Returns:
            dict of variables' name and value
        """
        try:
            self.log.debug('kernel: %s, original cell: %s', _nb_cron_kernel, _nb_cron_code)
            # check each line of code
            _nb_cron_code_cleaned = []
            for _nb_cron_code_line in _nb_cron_code.split("\n"):
                _nb_cron_code_line = _nb_cron_code_line.strip()
                # remove magic line
                if not _nb_cron_code_line.startswith("%"):
                    # remove val from scala spark code to mimic python code
                    if _nb_cron_kernel.startswith("spark") and _nb_cron_code_line.startswith("val "):
                        _nb_cron_code_line = _nb_cron_code_line[len("val "):]
                        # handle boolean case difference
                        _nb_cron_code_line = _nb_cron_code_line.replace("true", "True")
                        _nb_cron_code_line = _nb_cron_code_line.replace("false", "False")
                    _nb_cron_code_cleaned.append(_nb_cron_code_line)
            _nb_cron_code = "\n".join(_nb_cron_code_cleaned)
            self.log.debug('kernel: %s, processed cell: %s', _nb_cron_kernel, _nb_cron_code)

            # exec code to get variable names and values
            exec(_nb_cron_code)
            var = locals()
            var.pop("_nb_cron_code")  # remove method parameter
            var.pop("_nb_cron_code_cleaned")  # remove method parameter
            var.pop("_nb_cron_code_line")  # remove method parameter
            var.pop("_nb_cron_kernel")  # remove method parameter
            var.pop("self")  # remove self

            return var
        except NameError as err:
            self.log.error('[nb_cron] papermill extract parameters fail:\n%s', err)
            return {}
        except SyntaxError as err:
            self.log.error('[nb_cron] papermill extract parameters fail:\n%s', err)
            return {}

    def extract_papermill_parameters(self, path):
        """Process notebook to get papermill inputs

        Args:
            path: path to notebook

        Returns:
            successful - maps of notebook input/output abs path, env, kernel, parameters and status_code
            error - error message and  status_code
        """
        kernel = ""
        variables = {}
        try:
            import os
            notebook_input = os.path.abspath(path)
            notebook_output = notebook_input.replace(".ipynb", "_output.ipynb")
            notebook_log = notebook_input + ".cron.log"
            notebook_cwd = os.sep.join(notebook_input.split(os.sep)[:-1])
            notebook_string = open(notebook_input).read()
            notebook = json.loads(notebook_string)
            # get kernel
            env = ""
            conda_activate = ""
            kernel = notebook["metadata"]["kernelspec"]["name"]
            # check for conda env
            if kernel.startswith(self.conda_prefix_local) or kernel.startswith(self.conda_prefix_global):
                env_kernel = ""
                if kernel.startswith(self.conda_prefix_local):
                    env_kernel = kernel[len(self.conda_prefix_local):].split(self.conda_separator)
                elif kernel.startswith(self.conda_prefix_global):
                    env_kernel = kernel[len(self.conda_prefix_global):].split(self.conda_separator)
                env = env_kernel[0]
                kernel = self.conda_separator.join(env_kernel[1:])
                if env and kernel == 'py':
                    kernel = 'python3'
            else:
                # get current activated environment
                if 'CONDA_DEFAULT_ENV' in os.environ:
                    env = os.environ['CONDA_DEFAULT_ENV']
            # use CONDA_EXE instead of CONDA_PREFIX (assumes activate co-exists with conda)
            if 'CONDA_EXE' in os.environ:
                conda_activate = 'activate'.join(os.environ['CONDA_EXE'].rsplit('conda', 1))
                # defaults to empty if activate not found
                if not os.path.isfile(conda_activate):
                    conda_activate = ""
            self.log.debug('conda_activate: %s', conda_activate)

            # get parameters cell
            code = ""
            for cell in notebook["cells"]:
                if cell["cell_type"] == "code":
                    if "metadata" in cell and "tags" in cell["metadata"] and "parameters" in cell["metadata"]["tags"]:
                        code = "".join(cell["source"])
            variables = self.extract_variables(code, kernel)
        except FileNotFoundError as err:
            self.log.error('[nb_cron] Extract papermill parameters fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }
        except KeyError as err:
            self.log.error('[nb_cron] Extract papermill parameters fail:\n%s', err)
            return {
                "error": True,
                "message": u"{err}".format(err=err),
                "status_code": 422
            }

        return {
            "input": notebook_input,
            "output": notebook_output,
            "log": notebook_log,
            "cwd": notebook_cwd,
            "env": env,
            "activate": conda_activate,
            "kernel": kernel,
            "parameters": variables,
            "status_code": 200
        }



#############################################################################
# Copyright (c) 2021, nb_cron Contributors                                  #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################
# pylint: disable=W0221

# Tornado get and post handlers often have different args from their base class
# methods.

import json
import os
import re

from subprocess import Popen
from tempfile import TemporaryFile

from pkg_resources import parse_version
from notebook.utils import url_path_join as ujoin
from notebook.base.handlers import APIHandler
from tornado import web

from .jobmanager import JobManager

static = os.path.join(os.path.dirname(__file__), 'static')

NS = r'cron'


class JobBaseHandler(APIHandler):
    """
    Mixin for a job manager. Just maintains a reference to the
    'job_manager' which implements all of the cron functions.
    """

    @property
    def job_manager(self):
        """Return our job_manager instance"""
        return self.settings['job_manager']


class MainJobHandler(JobBaseHandler):
    """
    Handler for `GET /jobs` which lists the jobs.
    """

    @web.authenticated
    def get(self):
        self.finish(json.dumps(self.job_manager.list_jobs()))


class JobActionHandler(JobBaseHandler):
    """
    Handler for `POST /jobs/<job>/{delete,edit,create}`
    which performs the requested action on the cron job.
    """

    @web.authenticated
    def post(self, job, action):
        status = None

        if action == 'remove':
            data = self.job_manager.remove_job(int(job))
        elif action == 'edit':
            schedule = self.get_argument('schedule', default=None)
            command = self.get_argument('command', default=None)
            comment = self.get_argument('comment', default=None)
            if not schedule or not command:
                raise web.HTTPError(
                    status_code=422,
                    reason=u"Schedule/Command empty.",
                )
            data = self.job_manager.edit_job(int(job), schedule, command, comment)
        elif action == 'create':
            schedule = self.get_argument('schedule', default=None)
            command = self.get_argument('command', default=None)
            comment = self.get_argument('comment', default=None)
            if not schedule or not command:
                raise web.HTTPError(
                    status_code=422,
                    reason=u"Schedule/Command is empty.",
                )
            data = self.job_manager.create_job(schedule, command, comment)
            if 'error' not in data:
                status = 201  # CREATED

        # catch-all ok
        if 'error' in data:
            raise web.HTTPError(
                status_code=data['status_code'] or 400,
                reason=data['message'],
            )

        self.set_status(status or 200)
        self.finish(json.dumps(data))

class ScheduleActionHandler(JobBaseHandler):
    """
    Handler for `POST /schedule/{check}`
    which performs the requested action on the cron schedule.
    """

    @web.authenticated
    def post(self, action):
        if action == 'check':
            schedule = self.get_argument('schedule', default=None)
            if not schedule:
                raise web.HTTPError(
                    status_code=422,
                    reason=u"Schedule empty",
                )
            data = self.job_manager.check_schedule(schedule)

        # catch-all ok
        if 'error' in data:
            raise web.HTTPError(
                status_code=data['status_code'] or 400,
                reason=data['message'],
            )

        self.finish(json.dumps(data))

class NotebookActionHandler(JobBaseHandler):
    """
    Handler for `POST /notebook/{papermill}`
    which performs the requested action on the cron schedule.
    """

    @web.authenticated
    def post(self, action):
        if action == 'papermill':
            path = self.get_argument('path', default=None)
            if not path:
                raise web.HTTPError(
                    status_code=422,
                    reason=u"Path empty",
                )
            data = self.job_manager.extract_papermill_parameters(path)

        # catch-all ok
        if 'error' in data:
            raise web.HTTPError(
                status_code=data['status_code'] or 400,
                reason=data['message'],
            )

        self.finish(json.dumps(data))

# -----------------------------------------------------------------------------
# URL to handler mappings
# -----------------------------------------------------------------------------

_job_action_regex = r"(?P<action>create|edit|remove)"
_schedule_action_regex = r"(?P<action>check)"
_notebook_action_regex = r"(?P<action>papermill)"

# there is almost no text that is invalid, but no hyphens up front, please
# neither all these suspicious but valid characters...
_job_regex = r"(?P<job>-1|\d+)"

default_handlers = [
    (r"/jobs", MainJobHandler),
    (r"/jobs/%s/%s" % (_job_regex, _job_action_regex),
     JobActionHandler),
    (r"/schedule/%s" % (_schedule_action_regex),
     ScheduleActionHandler),
    (r"/notebook/%s" % (    _notebook_action_regex),
     NotebookActionHandler),
]


def load_jupyter_server_extension(nbapp):
    """Load the nbserver extension"""
    webapp = nbapp.web_app
    webapp.settings['job_manager'] = JobManager(parent=nbapp)

    base_url = webapp.settings['base_url']
    webapp.add_handlers(".*$", [
        (ujoin(base_url, NS, pat), handler)
        for pat, handler in default_handlers
    ])
    nbapp.log.info("[nb_cron] enabled")



#############################################################################
# Copyright (c) 2021, nb_cron Contributors                                  #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################
import sys

from notebook.nbextensions import enable_nbextension_python, install_nbextension_python
from notebook.serverextensions import toggle_serverextension_python
from notebook.tests.launchnotebook import NotebookTestBase
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.firefox.options import Options
from traitlets.config.loader import Config

import nb_cron


class NbCronNotebookTest(NotebookTestBase):
    def setUp(self):
        if 'nb_cron' not in sys.modules:
            sys.modules['nb_cron'] = nb_cron
        c = Config()
        c.NotebookApp.nbserver_extensions = {}
        c.NotebookApp.nbserver_extensions.update({'nb_cron': True})
        c.NotebookApp.allow_origin = '*'
        c.NotebookApp.allow_credentials = True
        c.NotebookApp.disable_check_xsrf = True
        self.config = c
        install_nbextension_python("nb_cron", user=True)
        enable_nbextension_python("nb_cron")
        toggle_serverextension_python("nb_cron", True)
        super(NbCronNotebookTest, self).setUp()
        self.__class__.notebook.init_server_extension_config()
        self.__class__.notebook.init_server_extensions()

        options = Options()
        options.add_argument("-headless")
        self.driver = webdriver.Firefox(options=options)

    def tearDown(self):
        self.driver.quit()
        super(NbCronNotebookTest, self).tearDown()

    def test_01_body(self):
        body = None
        try:
            self.driver.get(self.base_url() + '?token=' + self.token)
            self.driver.implicitly_wait(30)  # seconds
            body = self.driver.find_element_by_tag_name("body")
        except NoSuchElementException:
            pass
        self.assertIsNotNone(body)

    def test_02_cron_tab(self):
        cron_tab = None
        try:
            self.driver.get(self.base_url() + '?token=' + self.token)
            self.driver.implicitly_wait(30)  # seconds
            cron_tab = self.driver.find_element_by_id("cron_tab")
        except NoSuchElementException:
            pass
        self.assertIsNotNone(cron_tab)

    def test_03_job_list(self):
        job_list = None
        try:
            self.driver.get(self.base_url() + '?token=' + self.token)
            self.driver.implicitly_wait(30)  # seconds
            job_list = self.driver.find_element_by_id("job_list_body")
        except NoSuchElementException:
            pass
        self.assertIsNotNone(job_list)

    def test_04_create_job(self):
        job_list = None
        try:
            self.driver.get(self.base_url() + '?token=' + self.token)
            self.driver.implicitly_wait(30)  # seconds
            job_list = self.driver.find_element_by_id("job_list_body")
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "cron_tab"))).click()
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "new_job"))).click()
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "job_schedule"))).send_keys("* * * * *")
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "check_schedule"))).click()
        except NoSuchElementException:
            pass
        self.assertIsNotNone(job_list)

    def test_05_papermill_builder(self):
        job_list = None
        try:
            self.driver.get(self.base_url() + '?token=' + self.token)
            self.driver.implicitly_wait(30)  # seconds
            job_list = self.driver.find_element_by_id("job_list_body")
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "cron_tab"))).click()
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "new_job"))).click()
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "papermill_builder"))).click()
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "notebook_input"))).send_keys("tests/pyspark parameter test.ipynb")
            WebDriverWait(self.driver, 30).until(expected_conditions.element_to_be_clickable((By.ID, "inspect_notebook"))).click()
        except NoSuchElementException:
            pass
        self.assertIsNotNone(job_list)



import setuptools
import versioneer

setuptools.setup(
    name="nb_cron",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    url="https://github.com/alexanghh/nb_cron",
    author="alexanghh",
    author_email="alexanghh@gmail.com",
    description="Manage your crontab from the Jupyter Notebook",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'notebook>=4.3.1',
        'croniter>=1.0.13',
        'python-crontab>=2.5.1'
    ]
)
